
// Watchdog Test
// True or False: Windows has a watchdog timer that keeps kernels from running for more than a few seconds?
//  This is an incredibly simple kernel that just runs for a long long time...we'll see.

// This code has been modified from a template that carries the following message:
/*
 * Copyright 1993-2007 NVIDIA Corporation.  All rights reserved.
 *
 * NOTICE TO USER:
 *
 * This source code is subject to NVIDIA ownership rights under U.S. and
 * international Copyright laws.  Users and possessors of this source code
 * are hereby granted a nonexclusive, royalty-free license to use this code
 * in individual and commercial software.
 *
 * NVIDIA MAKES NO REPRESENTATION ABOUT THE SUITABILITY OF THIS SOURCE
 * CODE FOR ANY PURPOSE.  IT IS PROVIDED "AS IS" WITHOUT EXPRESS OR
 * IMPLIED WARRANTY OF ANY KIND.  NVIDIA DISCLAIMS ALL WARRANTIES WITH
 * REGARD TO THIS SOURCE CODE, INCLUDING ALL IMPLIED WARRANTIES OF
 * MERCHANTABILITY, NONINFRINGEMENT, AND FITNESS FOR A PARTICULAR PURPOSE.
 * IN NO EVENT SHALL NVIDIA BE LIABLE FOR ANY SPECIAL, INDIRECT, INCIDENTAL,
 * OR CONSEQUENTIAL DAMAGES, OR ANY DAMAGES WHATSOEVER RESULTING FROM LOSS
 * OF USE, DATA OR PROFITS,  WHETHER IN AN ACTION OF CONTRACT, NEGLIGENCE
 * OR OTHER TORTIOUS ACTION,  ARISING OUT OF OR IN CONNECTION WITH THE USE
 * OR PERFORMANCE OF THIS SOURCE CODE.
 *
 * U.S. Government End Users.   This source code is a "commercial item" as
 * that term is defined at  48 C.F.R. 2.101 (OCT 1995), consisting  of
 * "commercial computer  software"  and "commercial computer software
 * documentation" as such terms are  used in 48 C.F.R. 12.212 (SEPT 1995)
 * and is provided to the U.S. Government only as a commercial end item.
 * Consistent with 48 C.F.R.12.212 and 48 C.F.R. 227.7202-1 through
 * 227.7202-4 (JUNE 1995), all U.S. Government End Users acquire the
 * source code with only those rights set forth herein.
 *
 * Any use of this source code in individual and commercial software must
 * include, in the user documentation and internal comments to the code,
 * the above Disclaimer and U.S. Government End Users Notice.
 */

/* Template project which demonstrates the basics on how to setup a project 
* example application.
* Host code.
*/

// includes, system
#include <stdlib.h>
#include <stdio.h>
#include <windows.h>
#include <winbase.h>

// includes, project
#include <cutil.h>

// includes, kernels
#include <template_kernel.cu>

////////////////////////////////////////////////////////////////////////////////
// declaration, forward
void runTest( int argc, char** argv);


////////////////////////////////////////////////////////////////////////////////
// Program main
////////////////////////////////////////////////////////////////////////////////
int
main( int argc, char** argv) 
{
    runTest( argc, argv );

    CUT_EXIT(argc, argv);
}

////////////////////////////////////////////////////////////////////////////////
//! Run a simple test for CUDA
////////////////////////////////////////////////////////////////////////////////
void
runTest( int argc, char** argv) 
{

    CUT_DEVICE_INIT();

    unsigned int timer = 0;
    CUT_SAFE_CALL( cutCreateTimer( &timer ) );

    unsigned int num_threads = 32;
    unsigned int mem_size = sizeof( float ) * num_threads;

    // allocate device memory for result
    float* d_odata;
    CUDA_SAFE_CALL( cudaMalloc( (void**) &d_odata, mem_size ) );

    // setup execution parameters
    dim3  grid( 1, 1, 1 );
    dim3  threads( num_threads, 1, 1 );

	// message
	printf( "Watchdog Test Program. Run incresingly long but very simple\nkernels to see when Windows starts to complain.\n" );

	// do the kernel for a variety of op counts until Windows complains:
	for( int i = 4; i < 30; i += 1 )
	{

		CUT_SAFE_CALL( cutStartTimer( timer ) );

		// execute the kernel
		testKernel<<< grid, threads >>>( d_odata, i * 1e7 );

		// check if kernel execution generated and error
		cudaThreadSynchronize();
		cudaError_t err = cudaGetLastError();
		if( err != 0 )
		{
			printf( "Cuda Error: %s\n", cudaGetErrorString( err ) );
			break;
		}
		CUT_CHECK_ERROR("Kernel execution failed");

		// gather timing data:
		CUT_SAFE_CALL( cutStopTimer( timer ) );
		printf( "Launch Succeeded. Time: %f (s)\n", cutGetTimerValue( timer ) / 1000.0f );
		CUT_SAFE_CALL( cutResetTimer( timer ) );
		
		// we're going to wait for a second so Windows can get its act back together:
		Sleep( 1000 );

	}

    // cleanup memory
    CUDA_SAFE_CALL(cudaFree(d_odata));
    CUT_SAFE_CALL( cutDeleteTimer( timer));

}
